from karabo.middlelayer import AccessMode, Assignment, String
from .. import base_condition
from ..corrections.JungfrauCorrection import JungfrauGainMode


def settings_to_gain_mode(setting):
    gain_mode = JungfrauGainMode(setting)
    if gain_mode in (JungfrauGainMode.FIX_GAIN_1, JungfrauGainMode.FIX_GAIN_2):
        return 1
    else:
        return 0


def settings_to_gain_setting(setting):
    gain_mode = JungfrauGainMode(setting)
    if gain_mode is JungfrauGainMode.DYNAMIC_GAIN_HG0:
        return 1
    else:
        return 0


class JungfrauCondition(base_condition.ConditionBase):
    controlDeviceId = String(
        displayedName="Control device ID",
        assignment=Assignment.MANDATORY,
        accessMode=AccessMode.INITONLY,
    )

    @property
    def keys_to_get(self):
        return {
            self.controlDeviceId.value: [
                # cells: 1.0 or 16.0
                ("storageCells", "memoryCells", lambda n: n + 1),
                # observed voltages in parameter conditions:
                # 90.0
                # 180.0
                # 200.0
                # note: control device parameter is a vector
                ("vHighVoltage", "biasVoltage", lambda arr: round(arr[0], ndigits=0)),
                # observed integration times:
                # 9.999999747378752
                # 12.999999853491317
                # 49.99999873689376
                # 300.0000142492354
                # 349.9999875202775
                # 399.99998989515007
                # 500.00002374872565
                (
                    "exposureTime",
                    "integrationTime",
                    lambda n: round(n * 1e6, ndigits=0),
                ),
                # gain mode: omitted or 1.0
                ("settings", "gainMode", settings_to_gain_mode),
                # gain setting: 0.0 or 1.0 (derived from gain mode on device)
                ("settings", "gainSetting", settings_to_gain_setting),
            ]
        }
